#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Movable.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + VramRowSize * 1 + 32, "SCORE"); 
    PrintS(Vram + VramRowSize * 5 + 32, "HI-SCORE");
    PrintS(Vram + VramRowSize * 9 + 32, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 9 + 38, CurrentStage + 1);
    PrintS(Vram + VramRowSize * 12 + 32, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 22 + 33;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                Put2C(pVram, Char_Remain);
                pVram += 2;
                pVram = Put(pVram, Char_Space);
                pVram = Put(pVram, Char_Space);
                pVram += VramWidth - 2;
                pVram = Put(pVram, i + 0x10);
                pVram = Put(pVram, Char_Space);
            }
            else {
                do {
                    Put2C(pVram, Char_Remain);
                    pVram += 2;
                    --i;
                } while (i > 0);
            }
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 2 + 34, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 6 + 34, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize * 12 + 37, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 10 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 12, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 13, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	C
        0x1d, 0x2b, 0x22, 0x28, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x21, 0x28, 0x1d, 0x27, 
        0x1d, 0x21, 0x22, 0x1e, 
        //	R
        0x29, 0x24, 0x22, 0x28, 
        0x29, 0x20, 0x25, 0x2c, 
        0x29, 0x24, 0x2c, 0x1f, 
        0x21, 0x1e, 0x21, 0x22, 
        //	A
        0x1d, 0x2b, 0x2a, 0x1f, 
        0x29, 0x20, 0x1d, 0x2c, 
        0x29, 0x24, 0x22, 0x2c, 
        0x21, 0x1e, 0x1d, 0x22, 
        //	C
        0x1d, 0x2b, 0x22, 0x28, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x21, 0x28, 0x1d, 0x27, 
        0x1d, 0x21, 0x22, 0x1e, 
        //	K
        0x29, 0x20, 0x25, 0x24, 
        0x29, 0x28, 0x24, 0x1d, 
        0x29, 0x20, 0x2a, 0x1f, 
        0x21, 0x1e, 0x1d, 0x22, 
        //	Y
        0x29, 0x20, 0x29, 0x20, 
        0x21, 0x28, 0x2b, 0x1e, 
        0x1d, 0x29, 0x20, 0x1d, 
        0x1d, 0x21, 0x1e, 0x1d, 
    };
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 8 + 4;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 20 + 7, "PUSH TRIGGER BUTTON");
    PrintS(Vram + VramRowSize * 23 + 20, "INUFUTO 2023");
    // {
    //     ptr<byte> pVram;
    //     byte c;
    //     pVram = Vram;
    //     c = 0;
    //     repeat (64) {
    //         pVram = Put(pVram, c);
    //         ++c;
    //         if ((c & 0x0f) == 0) {
    //             pVram += VramRowSize - 16;
    //         }
    //     }
    // }
}
